# -*- coding: utf-8 -*-
#
#  LanControl2
#
#  Coded by dre (c) 2010 - 2014
#  Coding idea and design by dre
#  Coding support by Dr.Best
#  E-Mail: dre@dreambox-tools.info
#  Support: www.dreambox-tools.info
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Multimedia GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Multimedia GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Multimedia GmbH.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.

from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
from Components.Pixmap import Pixmap
from Components.config import config, ConfigSubsection, ConfigSubList, ConfigNumber, ConfigText, configfile, ConfigInteger, ConfigIP, NoSave, getConfigListEntry, ConfigYesNo # Dr. Best --> ConfigInteger, ConfigIP
from Plugins.Plugin import PluginDescriptor
from Screens.ChoiceBox import ChoiceBox
from Screens.HelpMenu import HelpableScreen
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools.Directories import fileExists
from wol import WakeOnLan
from dirSelect import dirSelectDlg

from enigma import eListboxPythonMultiContent, gFont, RT_HALIGN_RIGHT, loadPNG, eConsoleAppContainer

import os

# for localized messages
from . import _

config.plugins.lancontrol2 = ConfigSubsection()
config.plugins.lancontrol2.profile = ConfigSubList()
config.plugins.lancontrol2.autoquerystatus = ConfigYesNo(default = False)
config.plugins.lancontrol2.showinextensionsmenu = ConfigYesNo(default = False)
config.plugins.lancontrol2.privatekey = ConfigText(default="")
config.plugins.lancontrol2.profilecount = ConfigInteger(0) # Dr.Best --> Default Wert setzen, ausserdem auf ConfigInteger geaendert

def initProfileConfig():
	# hier baue ich eine neue CondigSubSection auf, und gebe die neue zurueck (siehe bei neu anlegen unten)
	s = ConfigSubsection()
	s.ipaddress =  ConfigText(default = "")
	s.macaddress = ConfigText(default = "")
	#s.status = NoSave(ConfigText(default = "yellow"))
	config.plugins.lancontrol2.profile.append(s)
	return s

def initConfig():
	count = config.plugins.lancontrol2.profilecount.value
	if count != 0:
		i = 0 # 0-basierender Index Aufpassen, wenn Du darauf zugreifen willst
		while i < count:
			initProfileConfig()
			i += 1

initConfig() # --> das ist wichtig, dass alle SubListItems initialisiert werden

##### Dr.Best Ende



class LanControlServerList(MenuList):
	def __init__(self, list):
		MenuList.__init__(self, list, False, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(25)

def LanControlServerListEntry(ipaddress, macaddress, status="yellow"):
	res = [ (ipaddress, macaddress, status) ]
	png = "/usr/share/enigma2/skin_default/buttons/button_%s.png" %(status)
	if fileExists(png):
		res.append(MultiContentEntryPixmapAlphaTest(pos=(2, 2), size=(20, 25), png=loadPNG(png)))
	res.append(MultiContentEntryText(pos=(20, 0), size=(280, 25), font=0, text=ipaddress))
	res.append(MultiContentEntryText(pos=(300, 0), size=(300, 25), font=0, text=macaddress, flags=RT_HALIGN_RIGHT))
	return res


class LanControlOverviewScreen(Screen,HelpableScreen):
	skin= """
		<screen name="LanControlOverviewScreen" position="center,center" size="620,560" title="LanControl2 - Overview">
			<widget name="menulist" position="10,10" size="600,475" scrollbarMode="showOnDemand" />
			<widget name="ButtonRedtext" position="30,480" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="10,490" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonGreentext" position="330,480" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="310,490" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonYellowtext" position="30,520" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonYellow" pixmap="skin_default/buttons/button_yellow.png" position="10,530" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonBluetext" position="330,520" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonBlue" pixmap="skin_default/buttons/button_blue.png" position="310,530" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""
	
	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close, _("Close plugin")),
			"ok":		(self.wakeUpServer, _("Start selected server")),
		}, -1)

		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
		{
			"menu":		(self.openPluginConfig, _("Open plugin config")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.removeServer,_("Remove selected server")),
			"green":	(self.addServer,_("Enter Add Server screen")),
			"yellow":	(self.queryStatus,_("Query server status")),
			"blue":		(self.changeServerState,_("Start/stop selected server")),
		}, -1)			

		self["ButtonRed"] = Pixmap()
		self["ButtonRedtext"] = Label(_("Remove selected server"))
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Add server"))
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowtext"] = Label(_("Query server status"))
		self["ButtonBlue"] = Pixmap()
		self["ButtonBluetext"] = Label(_("Start selected server"))
		self["ButtonBlue"].hide()
		self["ButtonBluetext"].hide()
		self.blueDisabled = True
		
		self.container = eConsoleAppContainer()
		self.container.appClosed.append(self.runFinished)
		self.container.dataAvail.append(self.dataAvail)
		
		self.pingcontainer = eConsoleAppContainer()
		self.pingcontainer.appClosed.append(self.pingFinished)
		self.pingcontainer.dataAvail.append(self.dataAvail)

		self.cmdList = []
		self.run = 0

		self.menulist = []
		self["menulist"] = LanControlServerList([])

		#menulist is built in getServers to use getServers for a refresh when returning to screen from addServer
		if config.plugins.lancontrol2.autoquerystatus.value == True:
			self.queryStatus()
		else:
			self.getServers()
			
		self["menulist"].onSelectionChanged.append(self.selectionChanged)

	def selectionChanged(self):
		index = self["menulist"].getSelectedIndex()
		if len(self.menulist):
			if self.menulist[index][0][2] == "red":
				self["ButtonBluetext"].setText(_("Start selected server"))
				self["ButtonBluetext"].show()
				self["ButtonBlue"].show()
			else:
				if config.plugins.lancontrol2.privatekey.value != "":
					self["ButtonBluetext"].setText(_("Stop selected server"))
				else:
					self["ButtonBluetext"].hide()
					self["ButtonBlue"].hide()

	def addServer(self):
		self.session.openWithCallback(self.getServers, addServerScreen, None)

	def getServers(self):
		self.menulist = []
		count = config.plugins.lancontrol2.profilecount.value
		if count != 0:
			for i in range(0,count):
				ipaddress = config.plugins.lancontrol2.profile[i].ipaddress.value
				macaddress = config.plugins.lancontrol2.profile[i].macaddress.value
				self.menulist.append(LanControlServerListEntry(ipaddress,macaddress))
			self["ButtonRed"].show()
			self["ButtonRedtext"].show()
			self["ButtonYellow"].show()
			self["ButtonYellowtext"].show()
			#self["ButtonBlue"].show()
			#self["ButtonBluetext"].show()
		else:
			self["ButtonRed"].hide()
			self["ButtonRedtext"].hide()
			self["ButtonYellow"].hide()
			self["ButtonYellowtext"].hide()
			self["ButtonBlue"].hide()
			self["ButtonBluetext"].hide()

		self.blueDisabled = True
		self["menulist"].l.setList(self.menulist)

	def removeServer(self):
		if len(self.menulist) > 0:
			sel = self["menulist"].getSelectedIndex()
			config.plugins.lancontrol2.profilecount.value = config.plugins.lancontrol2.profilecount.value - 1
			config.plugins.lancontrol2.profilecount.save()
			del config.plugins.lancontrol2.profile[sel]
			config.plugins.lancontrol2.profile.save()
			config.plugins.lancontrol2.save()
			configfile.save()
			self.getServers()

	def queryStatus(self):
		self.menulist = []
		self.ipaddress = ""
		self.macaddress = ""
		count = config.plugins.lancontrol2.profilecount.value
		if count != 0:
			self["menulist"].l.setList([LanControlServerListEntry(_("Querying server status..."), _("Please wait"),"")])		
			for i in range(0, count):
				cmd = "ping -c 1 -q %s" %(config.plugins.lancontrol2.profile[i].ipaddress.value)
				self.cmdList.append(cmd)
			self.pingcontainer.execute(self.cmdList[self.run])

			self["ButtonRed"].show()
			self["ButtonRedtext"].show()
			self["ButtonYellow"].show()
			self["ButtonYellowtext"].show()
			if config.plugins.lancontrol2.privatekey.value != "":
				self["ButtonBlue"].show()
				self["ButtonBluetext"].show()
		else:
			self["ButtonRed"].hide()
			self["ButtonRedtext"].hide()
			self["ButtonYellow"].hide()
			self["ButtonYellowtext"].hide()
			self["ButtonBlue"].hide()
			self["ButtonBluetext"].hide()

	def changeServerState(self):
		if len(self.menulist) > 0:
			index = self["menulist"].getSelectedIndex()
			if self.menulist[index][0][2] == "red":
				self.wakeUpServer(index)
			else:
				self.shutdownServer(index)

	def wakeUpServer(self, index):
		#mac-addresse fuer aktuell selektierten server aus der config lesen und an WakeOnLan uebergeben
		server = config.plugins.lancontrol2.profile[index].macaddress.value
		WakeOnLan(server)
		info = self.session.open(MessageBox,_("The command to boot the server has been sent"), MessageBox.TYPE_INFO, timeout=5)
		info.setTitle(_("LanControl2"))
		
	def shutdownServer(self, index):
		self.index = index
		self.session.openWithCallback(self.shutdownTypeSelected, ChoiceBox, list = [("Linux","Linux"), ("OSX","OSX")], title = _("Select Server-OS"), titlebartext = _("LanControl2"))
		
	def shutdownTypeSelected(self, answer):
		cmd = ""
		if answer != None:
			ip = self.menulist[self.index][0][0]
			if answer[1] == "Windows":
				cmd = "ssh root@%s -i %s 'shutdown -h now'" %(ip, config.plugins.lancontrol2.privatekey.value)
			else:
				cmd = "ssh root@%s -i %s 'shutdown -h now'" %(ip, config.plugins.lancontrol2.privatekey.value)
			self.cmdList.append(cmd)
			
			self.container.execute(self.cmdList[self.run])

	def openPluginConfig(self):
		self.session.open(PluginConfigScreen)
		
	def dataAvail(self, outputstring):
		print "[LanControl2 - Data available] %s" %(outputstring)

	def pingFinished(self, retval):
		print "[LandControl2 - Command executed] %d" %(retval)
		status = ""
		ipaddress = config.plugins.lancontrol2.profile[self.run].ipaddress.value
		macaddress = config.plugins.lancontrol2.profile[self.run].macaddress.value
		if retval != 0:
			status = "red"
			if self.blueDisabled:
				self.blueDisabled = False
				if self.run == 0:
					self["ButtonBluetext"].setText(_("Start selected server"))
					self["ButtonBluetext"].show()
					self["ButtonBlue"].show()
		elif retval == 0:
			status = "green"
			if self.blueDisabled:
				self.blueDisabled = False
				if self.run == 0:
					self["ButtonBluetext"].setText(_("Stop selected server"))
					self["ButtonBluetext"].show()
					self["ButtonBlue"].show()	
					
		self.menulist.append(LanControlServerListEntry(ipaddress,macaddress,status))		
		self.run += 1
		if self.run < len(self.cmdList):
			self.pingcontainer.execute(self.cmdList[self.run])
		else:
			self.run = 0
			self.cmdList = []
			self["menulist"].l.setList(self.menulist)	

	def runFinished(self, retval):
		print "[LandControl2 - Command executed] %d" %(retval)
		if retval != 0 and self.run>0:
			info = self.session.open(MessageBox,_("The command to shutdown the server has been sent but there was an error"), MessageBox.TYPE_ERROR, timeout=5)
			info.setTitle(_("LanControl2"))
		self.run += 1
		if self.run < len(self.cmdList):
			self.container.execute(self.cmdList[self.run])
		else:
			self.run = 0
			self.cmdList = []
			info = self.session.open(MessageBox,_("The command to shutdown the server has been sent"), MessageBox.TYPE_INFO, timeout=5)
			info.setTitle(_("LanControl2"))

class addServerScreen(Screen,HelpableScreen):
	skin="""
		<screen name="addServerScreen" position="center,center" size="620,320" title="LanControl2 - Add Server">
			<widget name="menulist" position="10,10" size="600,280" scrollbarMode="showOnDemand" />
			<widget name="ButtonGreentext" position="30,280" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="10,290" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonYellowtext" position="280,280" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonYellow" pixmap="skin_default/buttons/button_yellow.png" position="260,290" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""

	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close,_("Return to Overview screen")),
		}, -1)			

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveDevice,_("Save selected server")),
			"yellow":	(self.openSearchScreen,_("Open Search screen")),
		}, -1)			

		self.menulist = []
		self.serverlist = []
		self["menulist"] = LanControlServerList([])
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Save selected server"))
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowtext"] = Label(_("Search by IP address"))

		#menulist is built in readInterfaces to use readInterfaces for a refresh when returning to screen from searchMACAddress
		self.readInterfaces()

	def readInterfaces(self):
		self.menulist = []
		self.serverlist = []
		interfaces = {}

		try:
			fp = file('/proc/net/arp', 'r')
			interfaces = fp.readlines()
			fp.close()
		except:
			print "[LanControl2] arp - opening failed"
		
		for i in interfaces:
			split = i.strip().split(' ')	
			
			if len(split[0])==15:
				self.macaddress = "%s" % split[20]
			elif len(split[0])==14:
				self.macaddress = "%s" % split[21]
			elif len(split[0])==13:
				self.macaddress = "%s" % split[22]
			elif len(split[0])==12:
				self.macaddress = "%s" % split[23]
			elif len(split[0])==11:
				self.macaddress = "%s" % split[24]
			elif len(split[0])==10:
				self.macaddress = "%s" % split[25]
			elif len(split[0])==9:
				self.macaddress = "%s" % split[26]
			elif len(split[0])==8:
				self.macaddress = "%s" % split[27]
			elif len(split[0])==7:
				self.macaddress = "%s" % split[28]
			else:
				continue

			self.serverlist.append([split[0],self.macaddress])
			self.menulist.append(LanControlServerListEntry(split[0],self.macaddress,"green"))
			if len(self.serverlist) == 0:
				self["ButtonGreen"].hide()
				self["ButtonGreentext"].hide()
			else:
				self["ButtonGreen"].show()
				self["ButtonGreentext"].show()
		self["menulist"].l.setList(self.menulist)

	def saveDevice(self):
		# Dr.Best
		current = initProfileConfig() # neue ConfigSubsection in Object current
		index = self["menulist"].getSelectedIndex() # Dein Index
		# speicher die Werte in der ConfigSubsection
		current.ipaddress.value = self.serverlist[index][0]
		current.macaddress.value = self.serverlist[index][1]
		# speichern
		current.save()
		
		# erhoehe den Zaehler von profilecount um 1 und speichern
		config.plugins.lancontrol2.profilecount.value +=1
		config.plugins.lancontrol2.profilecount.save()
		# speicher die Config fuer das lancontrol2
		config.plugins.lancontrol2.save()
		# speicher configfile		
		configfile.save()

		info = self.session.open(MessageBox,_("Selected entry has been saved"), MessageBox.TYPE_INFO, timeout=5)
		info.setTitle(_("LanControl2"))

	def openSearchScreen(self):
		self.session.openWithCallback(self.readInterfaces, searchMACAddressScreen)

class searchMACAddressScreen(Screen,ConfigListScreen,HelpableScreen):
	skin="""
		<screen name="addServerScreen" position="center,center" size="520,60" title="LanControl2 - Search MACAddress">
			<widget name="config" position="10,10" size="500,30" scrollbarMode="showOnDemand"/>
			<widget name="ButtonGreentext" position="30,30" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="10,40" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""

	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.searchMACAddress,_("Search MACAddress")),
		}, -1)

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close,_("Return to AddServer screen")),
		}, -1)

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Search"))

		self.list = []
		self.ipbox = NoSave(ConfigIP(default = [0,0,0,0]))
		self.list.append(getConfigListEntry(_("Search MAC address with IP address:"), self.ipbox))

		ConfigListScreen.__init__(self, self.list)
		self["config"].setList(self.list)
	
	def searchMACAddress(self):
		self.pingresult = os.system("ping -c 1 -q %d.%d.%d.%d" % tuple(self.ipbox.value))
		if (int(self.pingresult)/256)==0:
			self.close()
		else:
			error = self.session.open(MessageBox,_("Couldn't ping IP address. Is the server running?"), MessageBox.TYPE_ERROR, timeout=5)
			error.setTitle(_("LanControl2"))

class PluginConfigScreen(Screen,ConfigListScreen,HelpableScreen):
	skin="""
		<screen name="PluginConfigScreen" position="center,center" size="620,145" title="LanControl2 - Plugin Config">
			<widget name="config" position="10,10" size="600,95" scrollbarMode="showOnDemand"/>
			<widget name="ButtonGreentext" position="30,105" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="10,115" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""

	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["OkCancelActions"]  = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":		(self.close,_("Return to Overview screen")),
			"ok":			(self.openDirSelect,_("Open directory selection")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveandclose,_("Save config and return to Overview screen")),
		}, -1)

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Save and Close"))

		self.list = []
		self.list.append(getConfigListEntry(_("Query status on plugin start"), config.plugins.lancontrol2.autoquerystatus))
		self.list.append(getConfigListEntry(_("Private key"), config.plugins.lancontrol2.privatekey))
		self.list.append(getConfigListEntry(_("Show plugin in extensions menu"), config.plugins.lancontrol2.showinextensionsmenu))

		ConfigListScreen.__init__(self, self.list)
		self["config"].setList(self.list)

	def saveandclose(self):
		for x in self.list:
			x[1].save()
		self.close()

	def cancel(self):
		for x in self["config"].list:
			x[1].cancel()
		self.close()
		
	def openDirSelect(self):
		selectedEntry = self["config"].getCurrentIndex()
		if selectedEntry == 1:
			self.session.openWithCallback(self.dirSelectDlgClosed, dirSelectDlg, "/etc/dropbear/", True)
		
	def dirSelectDlgClosed(self, directory):
		config.plugins.lancontrol2.privatekey.value = directory

def main(session, **kwargs):
	session.open(LanControlOverviewScreen)

def Plugins(**kwargs):
	list = [PluginDescriptor(name="LanControl2",
		description="Wake-On-LAN",where = [PluginDescriptor.WHERE_PLUGINMENU], icon="lancontrol2.png",fnc=main)]
	if config.plugins.lancontrol2.showinextensionsmenu.value:
		list.append(PluginDescriptor(name="LanControl2", description="Wake-On-LAN", where = [PluginDescriptor.WHERE_EXTENSIONSMENU], fnc=main))
	return list